/*:
 * @plugindesc v1.0 - Patches some behaviours in Yanfly's Event Chase Stealth plugin.
 * @author Caethyril
 *
 * @help Place this immediately below YEP_X_EventChaseStealth in the Plugin Manager.
 *
 * Overrides nonSeePlayer function of YEP_EventChasePlayer (requisite):
 *   - makes events able to chase you outside their original stealth region.
 * Overrides chaseConditions & fleeConditions functions of YEP_EventChaseStealth:
 *   - prevents events from stopping and staring at you when you're stealthed.
 *
 * https://forums.rpgmakerweb.com/index.php?threads/chase-event-stops-when-approaching-stealth-area.84308/
 */

// ~*~*~*~*~*~*~*~ YEP_EventChasePlayer stuff ~*~*~*~*~*~*~*~ //
//  fix for chasing stopping immediately when going stealthy  //

Game_Event.prototype.nonSeePlayer = function() {			// Override!
  if (Imported.YEP_X_EventChaseStealth) {
    if (this._alertLock <= 0 && this.meetStealthModeConditions()) {	// added _alertLock check
      this.stealthClearChaseSettings();
      this._stopCount = 0;
      return false;
    }
  }
  return !this._seePlayer;
};

// ~*~*~*~*~*~*~*~*~*~*~ YEP_X_EventChaseStealth stuff ~*~*~*~*~*~*~*~*~*~*~ //
// fix for automatic movement stopping when player is in range but stealthed //

Game_Event.prototype.chaseConditions = function(dis) {			// Override!
    if (this._alertLock <= 0 && this.meetStealthModeConditions()) {	// changed from $gamePlayer.isStealthMode()
      this.stealthClearChaseSettings();
      return false;
    }
    return Yanfly.ECS.Game_Event_chaseConditions.call(this, dis);
};

Game_Event.prototype.fleeConditions = function(dis) {			// Override!
    if (this._alertLock <= 0 && this.meetStealthModeConditions()) {	// changed from $gamePlayer.isStealthMode()
      this.stealthClearChaseSettings();
      return false;
    }
    return Yanfly.ECS.Game_Event_fleeConditions.call(this, dis);
};

// ~*~*~*~*~*~*~*~*~*~ end ~*~*~*~*~*~*~*~*~*~ //